#! /bin/sh

set -e

UNIT_DIR="/lib/systemd/system"
PKG_IGNORE="/var/lib/init-compat/package.ignore"

ignore_service() {
    unit=$1

    # Symlinks
    [ -L "$unit" ] && return 0

    # Units not installed with deb-system-helper
    DPKG_MAINTSCRIPT_PACKAGE=dummy-null-package deb-systemd-helper debian-installed $(basename ${unit}) || return 0

    # Units with matching named initscripts
    [ -f "/etc/init.d/$(basename ${unit} .service)" ] && return 0

    for f in $(basename ${unit}) $(grep "^Requires=" ${unit} | cut -f2 -d=) ; do
	case $f in
	    *.service)
		# DBus activation services
		grep -q "^Type=dbus" ${UNIT_DIR}/$f && return 0

		# Socket activation services
		grep -q "^Requires=.\+\.socket" ${UNIT_DIR}/$f && return 0
		;;
	    *)
	esac
    done

    return 1
}

parent_package() {
    unit=$1

    # Wildcard hack to work around merged-/usr-via-aliased-dirs breakage
    dpkg -S "*$unit" | cut -f1 -d:
}

ignore_package() {
    unit=$1
    pkg=$(parent_package $unit)

    # Packages listed in $PKG_IGNORE
    fgrep --no-messages --quiet --line-regexp ${pkg} $PKG_IGNORE && return 0

    # Packages that ship an initscript
    dpkg -L $pkg | grep -q /etc/init.d && return 0

    return 1
}

for unit in ${UNIT_DIR}/* ; do

    case $unit in
	*.path)
	;;
	*.socket)
	    # TODO: Handle with update-inetd?
	;;
	*.mount)
	;;
	*.target)
	;;
	*.timer)
	    # TODO: Handle with cron?
	;;
	*.wants)
	;;
	*@.service)
	;;
	*.service)
	    ignore_service $unit && continue
	    ignore_package $unit && continue

	    initscript=$(basename $unit .service)
	    echo No LSB initscript for $unit
	    echo Run /usr/share/init-compat/sysd2v.sh $unit
	    echo Review and edit generated initscript
	    echo Install in /etc/init.d/$initscript
	    echo chmod +x /etc/init.d/$initscript
	    echo update-rc.d $initscript defaults
	    echo
	    echo Or add \'$(parent_package $unit)\' to $PKG_IGNORE to ignore in future.
	    ;;
	*)
	    echo "Unhandled unit $unit"
	    ;;
    esac
done
